package moevie.db.jdbc;

import moevie.db.DataAccessException;
import moevie.db.dao.BeoordelingDAO;
import moevie.db.data.Beoordeling;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Implementatie van {@link BeoordelingDAO} met behulp van JDBC
 */
class JDBCBeoordelingDAO implements BeoordelingDAO {
    // Het veld url en de methode getConnection bevinden zich in elke
    // JDBC-DAO. Het zou geen slecht idee zijn om die
    // in een gemeenschappelijke bovenklasse te stoppen in plaats van
    // telkens te knippen en te plakken

    private String url;

    private Connection getConnection() throws SQLException {
        return DriverManager.getConnection(url);
    }

    public JDBCBeoordelingDAO(String url) {
        this.url = url;
    }

    @Override
    public List<Beoordeling> lijstBeoordelingen(int film) {
        try (Connection conn = getConnection();
        PreparedStatement ps = conn.prepareStatement(
                "SELECT Gebruiker, Commentaar, Sterren " +
                "FROM Beoordelingen " +
                "WHERE Film = ? AND Commentaar IS NOT NULL " +
                "ORDER BY Aangemaakt DESC, Gebruiker DESC"
        )) {
            ps.setInt (1, film);
            try (ResultSet rs = ps.executeQuery()) {
                ArrayList<Beoordeling> lijst =  new ArrayList<>();
                while (rs.next()) {
                    lijst.add(new Beoordeling(
                            film,
                            rs.getString("Gebruiker"),
                            rs.getString("Commentaar"),
                            rs.getInt ("Sterren")
                    ));
                }
                return lijst;
            }

        } catch (SQLException ex) {
            throw new DataAccessException("lijstBeoordelingen", ex);
        }
    }

    @Override
    public Beoordeling getBeoordeling(int film, String gebruiker) {
        try (Connection conn = getConnection();
             PreparedStatement ps = conn.prepareStatement(
                     "SELECT Commentaar, Sterren " +
                     "FROM Beoordelingen WHERE Film = ? AND Gebruiker = ?"
             )) {
            ps.setInt(1, film);
            ps.setString(2, gebruiker);
            try (ResultSet rs = ps.executeQuery()) {
                if (rs.next()) {
                    return new Beoordeling(
                            film, null,
                            rs.getString("Commentaar"),
                            rs.getInt ("Sterren")
                    );
                } else {
                    return null;
                }
            }
        } catch (SQLException ex) {
            throw new DataAccessException("getBeoordeling", ex);
        }
    }

    @Override
    public void markeerAlsGezien(String gebruiker, List<Integer> films) {
        try (Connection conn = getConnection();
        PreparedStatement ps = conn.prepareStatement(
                "INSERT INTO Beoordelingen(Film, Gebruiker) VALUES (?, ?)"
        )) {
            for (int film : films) {
                ps.setInt(1, film);
                ps.setString(2, gebruiker);
                ps.addBatch();
            }
            ps.executeBatch();
        } catch (SQLException ex) {
            // wanneer dezelfde film door dezelfde gebruiker twee keer gemarkeerd wordt, dan
            // willen we dit door de vingers zien. Vandaar de volgende if. De error code 23505
            // duidt op een 'duplicate key'. Zie documentatie van Apache Derby
            if (! ex.getSQLState().equals("23505")) {
                throw new DataAccessException("markeerAlsGezien", ex);
            }
        }
    }

    @Override
    public void nieuweBeoordeling(int film, String gebruiker, String commentaar, int sterren) {
		Beoordeling b = getBeoordeling(film, gebruiker);

		// Doe niets wanneer de gebruiker de film nog niet gezien heeft
		if (b == null) return;

		// Gooi exception wanneer de film al beoordeeld is door deze gebruiker
		if (b.isIngevuld())
			throw new DataAccessException("nieuweBeoordeling: al beoordeeld", null);

        try (Connection conn = getConnection();
             PreparedStatement ps = conn.prepareStatement(
                     "UPDATE Beoordelingen SET Commentaar=?, Sterren=? WHERE Film=? AND Gebruiker=?"
             )) {
            ps.setString(1, commentaar);
            ps.setInt(2, sterren);
            ps.setInt(3, film);
            ps.setString(4, gebruiker);

            ps.executeUpdate();
        } catch (SQLException ex) {
            throw new DataAccessException("nieuweBeoordeling", ex);
        }
    }

    @Override
    public void verwijderBeoordeling(int film, String gebruiker) {
        try (Connection conn = getConnection();
             PreparedStatement ps = conn.prepareStatement(
                     "DELETE FROM Beoordelingen WHERE Film=? AND Gebruiker=?"
             )) {
            ps.setInt(1, film);
            ps.setString(2, gebruiker);

            ps.executeUpdate();
        } catch (SQLException ex) {
            throw new DataAccessException("verwijderBeoordeling", ex);
        }

    }
}
