package moevie.db.jdbc;

import moevie.db.DataAccessException;
import moevie.db.dao.FilmDAO;
import moevie.db.data.Film;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Implementatie van {@link FilmDAO} met behulp van JDBC
 */
class JDBCFilmDAO implements FilmDAO {

    // Het veld url en de methode getConnection bevinden zich in elke
    // JDBC-DAO. Het zou geen slecht idee zijn om die
    // in een gemeenschappelijke bovenklasse te stoppen in plaats van
    // telkens te knippen en te plakken

    private String url;

    private Connection getConnection() throws SQLException {
        return DriverManager.getConnection(url);
    }

    public JDBCFilmDAO(String url) {
        this.url = url;
    }

    // Ingewikkelde select waarmee voor films ook het gemiddeld aantal sterren
    // wordt opgevraagd en of die film al dan niet gezien is door de gebruiker.
    //
    // Dit kan natuurlijk ook in afzonderlijke stukken worden opgesplitst, met het
    // nadeel dat er dan meer verbindingen met de databank moeten gemaakt worden
    private static final String FILM_SELECT =
            "SELECT ID, Titel, Hoofdrol, Eigenaar, " +
                    "AVG(CAST(Sterren AS DOUBLE)) AS Gemiddelde, " +
                    "SUM(CASE WHEN Gebruiker=? THEN 1 ELSE 0 END) AS Gezien " +
                    "FROM Films " +
                    "     LEFT JOIN Beoordelingen ON ID = Film " +
                    "GROUP BY ID, Titel, Hoofdrol, Eigenaar";

    /**
     * Zet het resultaat van een zoekopdracht om naar een film.
     */
    private static Film maakFilm(ResultSet rs) throws SQLException {
        return new Film(
                rs.getInt("ID"),
                rs.getString("Titel"),
                rs.getString("Hoofdrol"),
                rs.getString("Eigenaar"),
                rs.getBoolean("Gezien"),
                rs.getDouble("Gemiddelde") // 0 indien NULL
        );
    }

    public Film getFilm(int film, String gebruiker) {
        try (Connection conn = getConnection();
             PreparedStatement ps = conn.prepareStatement(
                     FILM_SELECT + " HAVING ID=?"
             )) {
            ps.setString(1, gebruiker); // parameter van FILM_SELECT
            ps.setInt(2, film);
            try (ResultSet rs = ps.executeQuery()) {
                if (rs.next()) {
                    return maakFilm(rs);
                } else {
                    return null;
                }
            }
        } catch (SQLException ex) {
            throw new DataAccessException("getFilm", ex);
        }
    }

    @Override
    public List<Film> lijstFilms(String gebruiker) {
        try (Connection conn = getConnection();
             PreparedStatement ps = conn.prepareStatement(
                     FILM_SELECT + ", Films.Aangemaakt " +
                             "ORDER BY Films.Aangemaakt DESC, Films.ID DESC"
             )) {
            ps.setString(1, gebruiker); // parameter van FILM_SELECT
            try (ResultSet rs = ps.executeQuery()) {
                ArrayList<Film> l = new ArrayList<>();
                while (rs.next()) {
                    l.add(maakFilm(rs));
                }
                return l;
            }
        } catch (SQLException ex) {
            throw new DataAccessException("lijstFilms", ex);
        }
    }

    @Override
    public int nieuweFilm(String titel, String eigenaar, String hoofdrol) {
        try (Connection conn = getConnection();
             PreparedStatement ps = conn.prepareStatement(
                     "INSERT INTO Films(Titel,Hoofdrol,Eigenaar) VALUES (?,?,?)", Statement.RETURN_GENERATED_KEYS
             )) {
            // doet de INSERT
            ps.setString(1, titel);
            ps.setString(2, hoofdrol);
            ps.setString(3, eigenaar);
            ps.executeUpdate();

            // haalt de ID op van de nieuwe film
            try (ResultSet rs = ps.getGeneratedKeys()) {
                rs.next();
                return rs.getInt(1);
            }
        } catch (SQLException ex) {
            throw new DataAccessException("nieuweFilm", ex);
        }
    }

    @Override
    public void verwijderFilm(int film) {
        try (Connection conn = getConnection();
             PreparedStatement ps = conn.prepareStatement(
                     "DELETE FROM Films WHERE ID = ?"
             )) {
            ps.setInt(1, film);
            ps.executeUpdate();
        } catch (SQLException ex) {
            throw new DataAccessException("verwijderFilm", ex);
        }

    }
}
